import type { IGomokuContext } from './context.type';
import { GomokuStateCache } from './state-cache';
import type { IGomokuState } from './state.type';
import type { IGomokuCandidateState, IGomokuPiece, IShapeScoreMap } from './types';
export interface IGomokuSolutionProps {
    MAX_ROW: number;
    MAX_COL: number;
    MAX_ADJACENT?: number;
    MAX_DEPTH_WIDE?: number;
    MAX_DEPTH_NARROW?: number;
    MAX_DEPTH_TIGHT?: number;
    MAX_DEPTH_DEEP?: number;
    MAX_DISTANCE_OF_NEIGHBOR?: number;
    POSSIBILITY_SEARCH_EQUIV_CANDIDATE?: number;
    scoreMap?: IShapeScoreMap;
}
export declare class GomokuSolution {
    readonly MAX_DEPTH_WIDE: number;
    readonly MAX_DEPTH_NARROW: number;
    readonly MAX_DEPTH_TIGHT: number;
    readonly MAX_DEPTH_DEEP: number;
    readonly POSSIBILITY_SEARCH_EQUIV_CANDIDATE: number;
    readonly context: IGomokuContext;
    readonly state: IGomokuState;
    readonly scoreMap: Readonly<IShapeScoreMap>;
    protected readonly _CANDIDATE_SCORE_WIDE_MIN: number;
    protected readonly _CANDIDATE_SCORE_NARROW_MIN: number;
    protected readonly _CANDIDATE_SCORE_TIGHT_MIN: number;
    protected readonly _CANDIDATE_SCORE_DEEP_MIN: number;
    protected readonly _CANDIDATE_SCORE_DEEP_MAX: number;
    protected readonly _cache: GomokuStateCache;
    protected readonly _candidatesList: IGomokuCandidateState[][];
    protected _bestMoveId: number;
    protected _mainPlayerId: number;
    constructor(props: IGomokuSolutionProps);
    init(pieces: ReadonlyArray<IGomokuPiece>): void;
    forward(r: number, c: number, playerId: number): void;
    revert(r: number, c: number): void;
    minimaxSearch(nextPlayer: number): [r: number, c: number];
    protected _alphaBeta(playerId: number, alpha: number, beta: number, cur: number, prevState: bigint): number;
    protected _searchWideSpace(playerId: number, alpha: number, beta: number, cur: number, prevState: bigint): number;
    protected _searchNarrowSpace(playerId: number, alpha: number, beta: number, cur: number, prevState: bigint): number;
    protected _searchTightSpace(playerId: number, alpha: number, beta: number, cur: number): number;
    protected _searchDeepSpace(playerId: number, cur: number): number;
    protected _score(nextPlayerId: number): number;
    /**
     * Simulated Annealing
     * @param candidates
     * @param size
     * @returns
     */
    protected _selectOneCandidate(candidates: IGomokuCandidateState[] | undefined, size: number): Iterable<IGomokuCandidateState>;
    protected _forward(posId: number, playerId: number): void;
    protected _revert(posId: number): void;
}
